package org.openmuc.jdlms.internal.lnassociation;

import java.io.IOException;
import java.util.Arrays;

import org.openmuc.jdlms.Authentication;
import org.openmuc.jdlms.internal.APdu;
import org.openmuc.jdlms.internal.ConnectionData;
import org.openmuc.jdlms.internal.DataDirectory;
import org.openmuc.jdlms.internal.asn1.axdr.AxdrType;
import org.openmuc.jdlms.internal.asn1.axdr.NullOutputStream;
import org.openmuc.jdlms.internal.asn1.cosem.COSEMpdu;
import org.openmuc.jdlms.internal.sessionlayer.ServerSessionLayer;

public class AssociationMessenger {

    private final ConnectionData connectionData;
    private final DataDirectory directory;
    private final ServerSessionLayer sessionLayer;

    public AssociationMessenger(ConnectionData connectionData, DataDirectory directory, ServerSessionLayer sessionLayer,
            Long connectionId) {
        this.connectionData = connectionData;
        this.directory = directory;
        this.sessionLayer = sessionLayer;
    }

    public void encodeAndSend(APdu aPdu) throws IOException {
        send(encodeApdu(aPdu));
    }

    public void send(byte[] data) throws IOException {
        this.sessionLayer.send(data, 0, data.length);
    }

    public COSEMpdu readNextCosemPdu() throws IOException {
        byte[] bytes = this.sessionLayer.readNextMessage();
        APdu apdu;

        Authentication authentication = connectionData.authentication;
        if (authentication.getCryptographicAlgorithm() != null) {
            if (connectionData.clientSystemTitle == null) {
                connectionData.clientSystemTitle = systemTitle();
            }
            apdu = APdu.decode(bytes, connectionData.clientSystemTitle, connectionData.frameCounter,
                    authentication.getGlobalEncryptionKey(), authentication.getAuthenticationKey());
        }
        else {
            apdu = APdu.decode(bytes);
        }

        COSEMpdu cosemPdu = apdu.cosemPdu;
        return cosemPdu;
    }

    public byte[] encodeApdu(APdu aPdu) throws IOException {
        // TODO: better user of the buffer
        byte[] buffer = new byte[0xFFFFF];
        int length = 0;

        Authentication authentication = connectionData.authentication;
        if (authentication.getCryptographicAlgorithm() != null) {

            length = aPdu.encode(buffer, connectionData.frameCounter++, systemTitle(),
                    authentication.getGlobalEncryptionKey(), authentication.getAuthenticationKey()); // TODO:
        }
        else {
            length = aPdu.encode(buffer);
        }

        return Arrays.copyOfRange(buffer, buffer.length - length, buffer.length);
    }

    public byte[] systemTitle() {
        return this.directory.getLogicalDeviceFor(this.sessionLayer.getLogicalDeviceId())
                .getLogicalDevice()
                .getSystemTitle();
    }

    public boolean pduSizeTooLarge(AxdrType actionResponse) throws IOException {
        return this.connectionData.clientMaxReceivePduSize != 0
                && pduSizeOf(actionResponse) >= this.connectionData.clientMaxReceivePduSize;
    }

    public static int pduSizeOf(AxdrType pdu) throws IOException {
        return pdu.encode(new NullOutputStream());
    }

}
