package org.openmuc.jdlms.sample.server;

import java.io.IOException;
import java.net.InetAddress;

import org.openmuc.jdlms.AccessResultCode;
import org.openmuc.jdlms.AttributeAddress;
import org.openmuc.jdlms.Authentication;
import org.openmuc.jdlms.Authentication.CryptographicAlgorithm;
import org.openmuc.jdlms.DlmsConnection;
import org.openmuc.jdlms.DlmsServer;
import org.openmuc.jdlms.HexConverter;
import org.openmuc.jdlms.LogicalDevice;
import org.openmuc.jdlms.MethodParameter;
import org.openmuc.jdlms.MethodResult;
import org.openmuc.jdlms.MethodResultCode;
import org.openmuc.jdlms.SetParameter;
import org.openmuc.jdlms.TcpConnectionBuilder;
import org.openmuc.jdlms.datatypes.DataObject;

public class SampleServer {
    private final static int PORT = 6789;
    private final static byte[] AUTHENTICATION_KEY = HexConverter
            .fromShortHexString("5468697349734150617373776f7264313233");
    private final static String MANUFACTURE_ID = "ISE";
    private final static long DEVICE_ID = 9999L;
    private final static String LOGICAL_DEVICE_ID = "L_D_I";
    private final static byte[] GLOBAL_ENCRYPTION_KEY = HexConverter
            .fromShortHexString("000102030405060708090a0b0c0d0e0f");
    private final static Authentication AUTHENTICATION = Authentication.newGmacAuthentication(AUTHENTICATION_KEY,
            GLOBAL_ENCRYPTION_KEY, CryptographicAlgorithm.AES_GMC_128);

    public static void main(String[] args) throws IOException {
        DlmsServer serverConnection;

        try {
            printServer("starting");
            LogicalDevice logicalDevice = new LogicalDevice(1, LOGICAL_DEVICE_ID, MANUFACTURE_ID, DEVICE_ID);
            logicalDevice.addRestriction(16, AUTHENTICATION);
            logicalDevice.registerCosemClass(new SampleClass());

            serverConnection = DlmsServer.tcpServerBuilder(PORT).registerLogicalDevice(logicalDevice).build();

            serverConnection.start();
            printServer("started");
        } catch (IOException e) {
            throw new IOException("DemoServer: " + e);
        }

        // runDemoClient();
        //
        // if (serverConnection != null) {
        // serverConnection.close();
        // }
        //
        // printServer("closed");
    }

    static void runDemoClient() throws IOException {
        TcpConnectionBuilder connectionBuilder = new TcpConnectionBuilder(InetAddress.getLocalHost())
                .setLogicalDeviceId(1)
                .setTcpPort(PORT)
                .setAuthentication(AUTHENTICATION)
                .setSystemTitle(MANUFACTURE_ID, DEVICE_ID);

        printClient("connecting to server");

        try (DlmsConnection client = connectionBuilder.buildLnConnection()) {
            printClient("connected");

            String obisCode = "0.0.0.2.1.255";
            MethodParameter param = new MethodParameter(99, obisCode, 2,
                    DataObject.newOctetStringData("Hello World".getBytes()));
            MethodResult result = client.action(param);
            printClient("Res code: " + result.getResultCode());

            if (result.getResultCode() == MethodResultCode.SUCCESS) {
                printClient("Res Data: " + result.getResultData() + " "
                        + new String((byte[]) result.getResultData().getRawValue()));
            }

            DataObject data = DataObject.newOctetStringData("Hello World".getBytes());
            SetParameter setParam = new SetParameter(new AttributeAddress(99, obisCode, 2), data);
            AccessResultCode setResult = client.set(setParam);
            printClient("Set Res:" + setResult);

            client.close();
            printClient("closed");

        } catch (IOException e) {
            System.err.println("DemoClient: exception:");
            e.printStackTrace();
            throw new IOException();
        }
    }

    private static void printClient(String message) {
        System.out.println("DemoClient: " + message);
    }

    private static void printServer(String message) {
        System.out.println("DemoServer: " + message);
    }
}
