package org.openmuc.jdlms;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.openmuc.jdlms.LnConformanceSetting.ACTION;
import static org.openmuc.jdlms.LnConformanceSetting.BLOCK_TRANSFER_WITH_GET_OR_READ;
import static org.openmuc.jdlms.LnConformanceSetting.EVENT_NOTIFICATION;
import static org.openmuc.jdlms.LnConformanceSetting.GET;
import static org.openmuc.jdlms.LnConformanceSetting.PRIORITY_MGMT_SUPPORTED;
import static org.openmuc.jdlms.LnConformanceSetting.SELECTIVE_ACCESS;
import static org.openmuc.jdlms.LnConformanceSetting.SET;

import java.util.Arrays;
import java.util.List;
import java.util.Set;

import org.junit.Test;
import org.openmuc.jdlms.internal.LnConformanceConverter;
import org.openmuc.jdlms.internal.asn1.cosem.Conformance;

public class LnConformanceSettingTest {

    // see 62056-5-3 © IEC:2013 p 127
    @Test
    public void test1() throws Exception {
        Conformance conformance = LnConformanceConverter.conformanceFor(PRIORITY_MGMT_SUPPORTED,
                BLOCK_TRANSFER_WITH_GET_OR_READ, GET, SET, SELECTIVE_ACCESS, EVENT_NOTIFICATION, ACTION);
        byte[] value = conformance.value;
        int bits = (value[2] & 0xFF) | value[1] << 8;

        assertEquals(0x00501F, bits);
    }

    @Test
    public void testSymmetry() throws Exception {
        List<LnConformanceSetting> settings = Arrays.asList(GET, SET);

        Set<LnConformanceSetting> conformanceSettingRes = LnConformanceConverter
                .conformanceSettingFor(LnConformanceConverter.conformanceFor(settings));

        for (LnConformanceSetting setting : settings) {
            assertTrue(conformanceSettingRes.contains(setting));
        }
    }

}
