package org.openmuc.jdlms.internal;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.openmuc.jdlms.CosemMethod;
import org.openmuc.jdlms.IllegalMethodAccessException;
import org.openmuc.jdlms.MethodResultCode;
import org.openmuc.jdlms.datatypes.DataObject;
import org.openmuc.jdlms.datatypes.DataObject.Type;

public class MethodAccessor {

    private final Method method;
    private final CosemMethod dlmsMethod;
    private final Type parameterType;
    private final Type returnType;
    private final int numOfParams;

    public MethodAccessor(Method method, CosemMethod dlmsMethod, Type parameterType, Type returnType) {
        this.method = method;
        this.dlmsMethod = dlmsMethod;
        this.parameterType = parameterType;
        this.returnType = returnType;
        this.numOfParams = this.method.getParameterTypes().length;
    }

    public CosemMethod getDlmsMethod() {
        return dlmsMethod;
    }

    public DataObject invoke(Object instance, DataObject argument, Long connectionId)
            throws IllegalMethodAccessException {

        Object result;
        try {
            Object[] methodArguments = buildMethodArguments(argument, connectionId);
            result = this.method.invoke(instance, methodArguments);

            if (result == null && this.returnType != null) {
                result = DataObject.newNullData();
            }

        } catch (IllegalAccessException | IllegalArgumentException e) {
            throw new IllegalMethodAccessException(MethodResultCode.OTHER_REASON);
        } catch (InvocationTargetException e) {
            if (!(e.getTargetException() instanceof IllegalMethodAccessException)) {
                throw new IllegalMethodAccessException(MethodResultCode.OTHER_REASON);
            }

            throw (IllegalMethodAccessException) e.getTargetException();
        }
        return (DataObject) result;
    }

    private Object[] buildMethodArguments(DataObject argument, Long connectionId) throws IllegalMethodAccessException {
        Object[] methodArguments = new Object[numOfParams];

        if (argument == null && this.parameterType == null) {
            if (this.numOfParams == 1) {
                methodArguments[0] = connectionId;
            }
        }
        else if (argument != null
                && (argument.getType() == this.parameterType || this.parameterType == Type.DONT_CARE)) {
            methodArguments[0] = argument;

            if (this.numOfParams == 2) {
                methodArguments[1] = connectionId;
            }
        }
        else {
            throw new IllegalMethodAccessException(MethodResultCode.TYPE_UNMATCHED);
        }
        return methodArguments;
    }

}
